/* eslint-disable no-case-declarations */
/* eslint-disable consistent-return */
import net from 'net';
import crypto from 'crypto';
import { promises as fs } from 'fs';
import Client from '../Client';

export default class CSCLI {
  public client: Client;

  public server: net.Server;

  #hmac: string;

  constructor(client: Client) {
    this.client = client;
    this.loadKeys();
    this.server = net.createServer((socket) => {
      socket.on('data', async (data) => {
        try {
          await this.handle(socket, data);
        } catch (err) {
          await this.client.util.handleError(err);
          socket.destroy();
        }
      });
    });
    this.init();
  }

  public async handle(socket: net.Socket, data: Buffer) {
    const parsed: { Username: string, Type: string, Message?: string, HMAC: string } = JSON.parse(data.toString().trim());
    let verificationParsed: any = parsed;
    delete verificationParsed.HMAC;
    verificationParsed = JSON.stringify(verificationParsed);
    const verification = this.verifyConnection(parsed.HMAC, verificationParsed);
    if (!verification) return socket.destroy();
    // FINISH VERIFICATION CHECKS
    switch (parsed.Type) {
      case 'lock':
        await this.client.util.accounts.lock(parsed.Username, this.client.user.id, { reason: 'Failed to accept Terms of Service.' });
        break;
      case 'ram':
        const mem = Number(await this.client.util.exec(`memory ${parsed.Username}`)) * 1000;
        const memoryConversion = mem / 1024 / 1024;
        socket.write(`${memoryConversion}\n`);
        socket.destroy();
        break;
      case 'storage':
        const res = await this.client.redis.get(`storage-${parsed.Username}`) ? Number(await this.client.redis.get(`storage-${parsed.Username}`)) : '0';
        socket.write(`${res}\n`);
        socket.destroy();
        break;
      default:
        socket.destroy();
        break;
    }
  }

  public verifyConnection(key: string, data: any): boolean {
    const hmac = crypto.createHmac('sha256', this.#hmac);
    hmac.update(data);
    const computed = hmac.digest('hex');
    if (computed === key) return true;
    return false;
  }

  public async loadKeys() {
    const key = await fs.readFile('/etc/cscli.conf', { encoding: 'utf8' });
    this.#hmac = key.toString().trim();
  }

  public init() {
    this.server.on('error', (err) => {
      this.client.util.handleError(err);
    });
    this.server.listen(8124, () => {
      this.client.signale.success('TCP socket is now listening for connections.');
    });
  }
}
